#ifndef __PLUGIN_H_
#define __PLUGIN_H_

#include "plugapi.h"

/**
 * plugin_load
 *
 *   Loads the plugin from the file specified by 'filename'. If 'lazy' is 
 *   non-zero, undefined symbols in the loaded plugin will be resolved 
 *   when they are used, otherwise immediately.
 *
 * Parameters:
 *
 *   filename - the plugin filename
 *   lazy     - controls resolution of undefined symbols
 *
 * Returns: the plugin handle
 */
struct mhandle *plugin_load(const char *filename, int lazy);

/**
 * plugin_data
 * 
 *   Returns the plugin info struct (see plugapi.h). As the handle returned 
 *   from plugin_open() is opaque, this function must be used to access the
 *   plugin data, flags and API.
 *
 *   Note: perhaps this function should be named something else...
 *
 * Parameters:
 *
 *   handle - the plugin handle returned from plugin_load()
 *
 * Returns: the plugin info struct
 */
struct motion_plugin *plugin_data(struct mhandle *handle);

/**
 * plugin_close
 *
 *   Closes/unloads the plugin specified by the handle.
 *
 * Parameters:
 *
 *   handle - the plugin handle returned from plugin_load()
 *
 * Returns: nothing
 */
void plugin_close(struct mhandle *handle);

/**
 * plugin_resolve
 *
 *   Resolves a symbol in the plugin specified by the handle. The address of
 *   the symbol (be it a function, a struct or anything else) is stored in the
 *   pointer pointed to by 'address'.
 *
 * Parameters:
 *
 *   handle  - the plugin handle returned from plugin_load()
 *   symbol  - the symbol to resolve, e.g. the name of a function
 *   address - pointer to a pointer that will receive the symbol address
 *
 * Returns: non-zero (true) on success, zero (false) on failure
 */
int plugin_resolve(struct mhandle *handle, const char *symbol, void **address);

#endif /* __PLUGIN_H_ */

